#!/usr/bin/python
#License: GPL Version 2.0
#author: Dave Aitel
#http://www.immunitysec.com/ for more information

import time
import getopt
import os
import sys
import md5
import urllib

hashdburl="https://www.immunitysec.com/hashdb/gethash.py"

#md5 sums a file
def getmd5sum(filename):
    m=md5.new()
    try:
        f=open(filename)
    except:
        return ""
    
    #insert error checking here
    s=f.read()
    f.close()
    m.update(s)
    return m.hexdigest()

#returns a dictionary that is the opposite of urllib.urlencode()
def urldecode(values):
    #print "urldecodeing "+values
    result={}
    valueslist=values.split("&")
    for v in valueslist:
        if len(v.split("="))!=2:
            print "Invalid response from server: "+values
            return result
        
        left=v.split("=")[0]
        right=v.split("=")[1]
        result[left]=right
    return result

def doCheck(filename,printresult=0):
    #now we retrieve the values from the remote web server, which
    #uses it's own mySQL database
    filebase=os.path.basename(filename)
    argdict={}
    argdict["filename"]=filebase
    arguments=urllib.urlencode(argdict)

    response=urllib.urlopen(hashdburl,arguments)
    if printresult:
        
        print response.read()
        sys.exit(0)
        
    hash=getmd5sum(filename)
    foundhash=0
    line=response.readline()
    havehashes=0
    while line!="":
        lineDict=urldecode(line)
        havehashes=1
        if lineDict["hash"]==hash:
            #print "Found our hash"
            return lineDict["description"]
        line=response.readline()
            
    if havehashes:
        #print "Returning Trojan"
        #print "Hash of filename %s=%s" %(filename,hash)
        return "trojan"
    else:
        #print "returning nohashes"
        return "nohashes"
        




#prints usage menu and then exits
def usage():
    print """
    usage: ./checkfile.py [options]
    options include:
    -f n    where n is the filename
    -p      print results of search without a local file to hash
    -u n    where n is the hashdb host (default is www.immunitysec.com)
    -h      this message
    """
    sys.exit(1)


if __name__=='__main__':

    
    filename=""

    printresult=0

    ourResultDict={}
    ourResultDict["clean"]="This file was found and is clean."
    ourResultDict["trojan"]="This file was found and is a trojan!"
    ourResultDict["vulnerable"]="This file was found and is vulnerable!"
    ourResultDict["nohashes"]="There were no hashes in the database for that file."
    
    try:
        (opts,args)=getopt.getopt(sys.argv[1:],"f:hu:p")
    except getopt.GetoptError:
        #print help
        usage()

    for o,a in opts:
        #print "option: "+o
        if o in ["-f"]:
            filename=a
        elif o in ["-u"]:
            hashdburl=a
        elif o in ["-h"]:
            usage()
        elif o in ["-p"]:
            printresult=1
        else:
            print "Option %s not recognized!" % o


    if filename=="":
        print "I need a filename to check"
        usage()

    
    if not printresult and not os.path.isfile(filename):
        print "Could not find the file!"
        sys.exit(1)

    #did you know at one point the east india company controlled half the world's commerce and 1/3 the people? How amazing is that?

    
    value=doCheck(filename,printresult)

    print ourResultDict[value]

